@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_alpha_instr.c,v 1.7 2006/02/09 22:40:27 debug Exp $
+ *  $Id: cpu_alpha_instr.c,v 1.11 2006/06/03 06:46:44 debug Exp $
  *
  *  Alpha instructions.
  *
@@ -749,14 +749,10 @@ X(to_be_translated)
 {
 	uint64_t addr, low_pc;
 	uint32_t iword;
-	struct alpha_vph_page *vph_p;
 	unsigned char *page;
 	unsigned char ib[4];
 	void (*samepage_function)(struct cpu *, struct alpha_instr_call *);
 	int opcode, ra, rb, func, rc, imm, load, loadstore_type, fp, llsc;
-#ifdef DYNTRANS_BACKEND
-	int simple = 0;
-#endif
 
 	/*  Figure out the (virtual) address of the instruction:  */
 	low_pc = ((size_t)ic - (size_t)cpu->cd.alpha.cur_ic_page)
@@ -768,16 +764,18 @@ X(to_be_translated)
 	cpu->pc = addr;
 
 	/*  Read the instruction word from memory:  */
-	if ((addr >> ALPHA_TOPSHIFT) == 0) {
-		vph_p = cpu->cd.alpha.vph_table0[(addr >>
-		    ALPHA_LEVEL0_SHIFT) & 8191];
-		page = vph_p->host_load[(addr >> ALPHA_LEVEL1_SHIFT) & 8191];
-	} else if ((addr >> ALPHA_TOPSHIFT) == ALPHA_TOP_KERNEL) {
-		vph_p = cpu->cd.alpha.vph_table0_kernel[(addr >>
-		    ALPHA_LEVEL0_SHIFT) & 8191];
-		page = vph_p->host_load[(addr >> ALPHA_LEVEL1_SHIFT) & 8191];
-	} else
-		page = NULL;
+	{
+		const uint32_t mask1 = (1 << DYNTRANS_L1N) - 1;
+		const uint32_t mask2 = (1 << DYNTRANS_L2N) - 1;
+		const uint32_t mask3 = (1 << DYNTRANS_L3N) - 1;
+		uint32_t x1 = (addr >> (64-DYNTRANS_L1N)) & mask1;
+		uint32_t x2 = (addr >> (64-DYNTRANS_L1N-DYNTRANS_L2N)) & mask2;
+		uint32_t x3 = (addr >> (64-DYNTRANS_L1N-DYNTRANS_L2N-
+		    DYNTRANS_L3N)) & mask3;
+		struct DYNTRANS_L2_64_TABLE *l2 = cpu->cd.alpha.l1_64[x1];
+		struct DYNTRANS_L3_64_TABLE *l3 = l2->l3[x2];
+		page = l3->host_load[x3];
+	}
 
 	if (page != NULL) {
 		/*  fatal("TRANSLATION HIT!\n");  */
@@ -791,14 +789,9 @@ X(to_be_translated)
 		}
 	}
 
-#ifdef HOST_LITTLE_ENDIAN
-	iword = *((uint32_t *)&ib[0]);
-#else
-	iword = ib[0] + (ib[1]<<8) + (ib[2]<<16) + (ib[3]<<24);
-#endif
-
-	/*  fatal("{ Alpha: translating pc=0x%016llx iword=0x%08x }\n",
-	    (long long)addr, (int)iword);  */
+	/*  Alpha instruction words are always little-endian. Convert
+	    to host order:  */
+	iword = LE32_TO_HOST( *((uint32_t *)&ib[0]) );
 
 
 #define DYNTRANS_TO_BE_TRANSLATED_HEAD
@@ -917,6 +910,7 @@ X(to_be_translated)
 		case 0x02: ic->f = instr(s4addl); break;
 		case 0x09: ic->f = instr(subl); break;
 		case 0x0b: ic->f = instr(s4subl); break;
+		case 0x0f: ic->f = instr(cmpbge); break;
 		case 0x12: ic->f = instr(s8addl); break;
 		case 0x1b: ic->f = instr(s8subl); break;
 		case 0x1d: ic->f = instr(cmpult); break;
@@ -935,6 +929,7 @@ X(to_be_translated)
 		case 0x82: ic->f = instr(s4addl_imm); break;
 		case 0x89: ic->f = instr(subl_imm); break;
 		case 0x8b: ic->f = instr(s4subl_imm); break;
+		case 0x8f: ic->f = instr(cmpbge_imm); break;
 		case 0x92: ic->f = instr(s8addl_imm); break;
 		case 0x9b: ic->f = instr(s8subl_imm); break;
 		case 0x9d: ic->f = instr(cmpult_imm); break;
