@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_i960.c,v 1.4 2005/11/13 00:14:07 debug Exp $
+ *  $Id: cpu_i960.c,v 1.9 2006/06/16 18:31:25 debug Exp $
  *
  *  Intel i960 CPU emulation.
  */
@@ -148,6 +148,33 @@ void i960_cpu_register_match(struct machine *m, char *name,
 }
 
 
+/*
+ *  i960_cpu_tlbdump():
+ *
+ *  Called from the debugger to dump the TLB in a readable format.
+ *  x is the cpu number to dump, or -1 to dump all CPUs.
+ *
+ *  If rawflag is nonzero, then the TLB contents isn't formated nicely,
+ *  just dumped.
+ */
+void i960_cpu_tlbdump(struct machine *m, int x, int rawflag)
+{
+}
+
+
+/*
+ *  i960_cpu_gdb_stub():
+ *
+ *  Execute a "remote GDB" command. Returns a newly allocated response string
+ *  on success, NULL on failure.
+ */
+char *i960_cpu_gdb_stub(struct cpu *cpu, char *cmd)
+{
+	fatal("i960_cpu_gdb_stub(): TODO\n");
+	return NULL;
+}
+
+
 /*
  *  i960_cpu_interrupt():
  */
@@ -189,7 +216,7 @@ static void print_spaces(int len) { int i; debug(" "); for (i=0; i<16-len/2*5;
  *  cpu->pc for relative addresses.
  */
 int i960_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
-	int running, uint64_t dumpaddr, int bintrans)
+	int running, uint64_t dumpaddr)
 {
 	uint64_t offset;
 	int len = 0;
