@@ -25,59 +25,18 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory_mips.c,v 1.2 2005/11/30 16:23:09 debug Exp $
+ *  $Id: memory_mips.c,v 1.8 2006/06/24 21:47:23 debug Exp $
  *
  *  MIPS-specific memory routines. Included from cpu_mips.c.
+ *
+ *  NOTE: The cache emulation code (ifdef ENABLE_CACHE_EMULATION) is old
+ *        and doesn't work with dyntrans. TODO: rewrite this.
  */
 
 #include <sys/types.h>
 #include <sys/mman.h>
 
 
-/*
- *  insert_into_tiny_cache():
- *
- *  If the tiny cache is enabled (USE_TINY_CACHE), then this routine inserts
- *  a vaddr to paddr translation first in the instruction (or data) tiny
- *  translation cache.
- */
-static void insert_into_tiny_cache(struct cpu *cpu, int instr, int writeflag,
-	uint64_t vaddr, uint64_t paddr)
-{
-#ifdef USE_TINY_CACHE
-	int wf = 1 + (writeflag == MEM_WRITE);
-
-	if (cpu->machine->bintrans_enable)
-		return;
-
-	paddr &= ~0xfff;
-	vaddr >>= 12;
-
-	if (instr) {
-		/*  Code:  */
-		memmove(&cpu->cd.mips.translation_cache_instr[1],
-		    &cpu->cd.mips.translation_cache_instr[0],
-		    sizeof(struct translation_cache_entry) *
-		    (N_TRANSLATION_CACHE_INSTR - 1));
-
-		cpu->cd.mips.translation_cache_instr[0].wf = wf;
-		cpu->cd.mips.translation_cache_instr[0].vaddr_pfn = vaddr;
-		cpu->cd.mips.translation_cache_instr[0].paddr = paddr;
-	} else {
-		/*  Data:  */
-		memmove(&cpu->cd.mips.translation_cache_data[1],
-		    &cpu->cd.mips.translation_cache_data[0],
-		    sizeof(struct translation_cache_entry) *
-		    (N_TRANSLATION_CACHE_DATA - 1));
-
-		cpu->cd.mips.translation_cache_data[0].wf = wf;
-		cpu->cd.mips.translation_cache_data[0].vaddr_pfn = vaddr;
-		cpu->cd.mips.translation_cache_data[0].paddr = paddr;
-	}
-#endif
-}
-
-
 /*
  *  memory_cache_R3000():
  *
@@ -118,13 +77,6 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 	hit = (rp[cache_line].tag_valid & R3000_TAG_VALID) &&
 	    (rp[cache_line].tag_paddr == (paddr & tag_mask));
 
-#ifdef ENABLE_INSTRUCTION_DELAYS
-	if (!hit)
-		cpu->cd.mips.instruction_delay +=
-		    cpu->cd.mips.cpu_type.instrs_per_cycle
-		    * cpu->cd.mips.cache_miss_penalty[which_cache];
-#endif
-
 	/*
 	 *  The cache miss bit is only set on cache reads, and only to the
 	 *  data cache. (?)
@@ -252,15 +204,6 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 		}
 	}
 
-	/*  Run instructions from the right host page:  */
-	if (cache == CACHE_INSTRUCTION) {
-		memblock = memory_paddr_to_hostaddr(mem, paddr, writeflag);
-		if (memblock != NULL) {
-			cpu->cd.mips.pc_last_host_4k_page = memblock +
-			    (paddr & ((1 << BITS_PER_MEMBLOCK) - 1) & ~0xfff);
-		}
-	}
-
 	/*  Write-through! (Write to main memory as well.)  */
 	if (writeflag == MEM_READ || cache_isolated)
 		return 1;
@@ -283,13 +226,6 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 		& ~cpu->cd.mips.cache_mask[which_cache])
 	    == (paddr & ~(cpu->cd.mips.cache_mask[which_cache]));
 
-#ifdef ENABLE_INSTRUCTION_DELAYS
-	if (!hit)
-		cpu->cd.mips.instruction_delay +=
-		    cpu->cd.mips.cpu_type.instrs_per_cycle
-		    * cpu->cd.mips.cache_miss_penalty[which_cache];
-#endif
-
 	/*
 	 *  The cache miss bit is only set on cache reads, and only to the
 	 *  data cache. (?)
@@ -314,10 +250,10 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 
 	/*  Data cache isolated?  Then don't access main memory:  */
 	if (cache_isolated) {
-		/*  debug("ISOLATED write=%i cache=%i vaddr=%016llx "
-		    "paddr=%016llx => addr in cache = 0x%lx\n",
-		    writeflag, cache, (long long)vaddr,
-		    (long long)paddr, addr);  */
+		/*  debug("ISOLATED write=%i cache=%i vaddr=%016"PRIx64" "
+		    "paddr=%016"PRIx64" => addr in cache = 0x%lx\n",
+		    writeflag, cache, (uint64_t) vaddr,
+		    (uint64_t) paddr, addr);  */
 
 		if (writeflag==MEM_READ) {
 			for (i=0; i<len; i++)
@@ -341,41 +277,32 @@ int memory_cache_R3000(struct cpu *cpu, int cache, uint64_t paddr,
 }
 
 
-#define TRANSLATE_ADDRESS	translate_address_mmu3k
+#define TRANSLATE_ADDRESS	translate_v2p_mmu3k
 #define	V2P_MMU3K
 #include "memory_mips_v2p.c"
 #undef TRANSLATE_ADDRESS
 #undef V2P_MMU3K
 
-#define TRANSLATE_ADDRESS	translate_address_mmu8k
+#define TRANSLATE_ADDRESS	translate_v2p_mmu8k
 #define	V2P_MMU8K
 #include "memory_mips_v2p.c"
 #undef TRANSLATE_ADDRESS
 #undef V2P_MMU8K
 
-#define TRANSLATE_ADDRESS	translate_address_mmu10k
+#define TRANSLATE_ADDRESS	translate_v2p_mmu10k
 #define	V2P_MMU10K
 #include "memory_mips_v2p.c"
 #undef TRANSLATE_ADDRESS
 #undef V2P_MMU10K
 
 /*  Almost generic  :-)  */
-#define TRANSLATE_ADDRESS	translate_address_mmu4100
+#define TRANSLATE_ADDRESS	translate_v2p_mmu4100
 #define	V2P_MMU4100
 #include "memory_mips_v2p.c"
 #undef TRANSLATE_ADDRESS
 #undef V2P_MMU4100
 
-#define TRANSLATE_ADDRESS	translate_address_generic
+#define TRANSLATE_ADDRESS	translate_v2p_generic
 #include "memory_mips_v2p.c"
 
 
-#ifndef EXPERIMENTAL_NEWMIPS
-
-#define MEMORY_RW	mips_memory_rw
-#define MEM_MIPS
-#include "../memory_rw.c"
-#undef MEM_MIPS
-#undef MEMORY_RW
-
-#endif
