@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_cons.c,v 1.33 2006/02/09 20:02:59 debug Exp $
+ *  $Id: dev_cons.c,v 1.36 2006/05/06 08:42:49 debug Exp $
  *  
  *  A simple console device, useful for simple tests.
  *
@@ -47,6 +47,8 @@
 #include "memory.h"
 #include "misc.h"
 
+#include "testmachine/dev_cons.h"
+
 
 #define	CONS_TICK_SHIFT		14
 
@@ -76,9 +78,12 @@ DEVICE_ACCESS(cons)
 
 	/*  Exit the emulator:  */
 	if (relative_addr == DEV_CONS_HALT) {
-		cpu->running = 0;
-		cpu->machine->exit_without_entering_debugger = 1;
-		return 1;
+		/*  cpu->running = 0;
+		    cpu->machine->exit_without_entering_debugger = 1;
+		    return 1;  */
+		/*  TODO: this doesn't work yet. for now, let's
+		    simply use exit()  */
+		exit(1);
 	}
 
 	if (writeflag == MEM_WRITE) {
@@ -145,7 +150,7 @@ DEVINIT(cons)
 	    devinit->addr, DEV_CONS_LENGTH, dev_cons_access, d,
 	    DM_DEFAULT, NULL);
 	machine_add_tickfunction(devinit->machine, dev_cons_tick,
-	    d, CONS_TICK_SHIFT);
+	    d, CONS_TICK_SHIFT, 0.0);
 
 	/*  NOTE: Ugly cast into pointer  */
 	devinit->return_ptr = (void *)(size_t)d->console_handle;
