@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_fb.c,v 1.116 2006/02/05 10:26:36 debug Exp $
+ *  $Id: dev_fb.c,v 1.118 2006/03/04 12:38:47 debug Exp $
  *  
  *  Generic framebuffer device.
  *
@@ -513,7 +513,10 @@ void dev_fb_tick(struct cpu *cpu, void *extra)
 #endif
 
 	if (d->update_x2 != -1) {
-		int y, addr, addr2, q = d->vfb_scaledown;
+#ifdef WITH_X11
+		int y;
+#endif
+		int addr, addr2, q = d->vfb_scaledown;
 
 		if (d->update_x1 >= d->visible_xsize)
 			d->update_x1 = d->visible_xsize - 1;
@@ -863,7 +866,7 @@ struct vfb_data *dev_fb_init(struct machine *machine, struct memory *mem,
 	memory_device_register(mem, name2, baseaddr, size, dev_fb_access,
 	    d, flags, d->framebuffer);
 
-	machine_add_tickfunction(machine, dev_fb_tick, d, FB_TICK_SHIFT);
+	machine_add_tickfunction(machine, dev_fb_tick, d, FB_TICK_SHIFT, 0.0);
 	return d;
 }
 
