@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_kn230.c,v 1.13 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_kn230.c,v 1.15 2006/02/09 20:02:59 debug Exp $
  *  
  *  DEC MIPSMATE 5100 (KN230) stuff.
  */
@@ -47,9 +47,7 @@
 /*
  *  dev_kn230_access():
  */
-int dev_kn230_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(kn230)
 {
 	struct kn230_csr *d = extra;
 	uint64_t idata = 0, odata = 0;
@@ -85,10 +83,7 @@ int dev_kn230_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_kn230():
- */
-int devinit_kn230(struct devinit *devinit)
+DEVINIT(kn230)
 {
 	struct kn230_csr *d;
 
