@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ns16550.c,v 1.51 2006/02/18 13:42:39 debug Exp $
+ *  $Id: dev_ns16550.c,v 1.54 2006/04/06 18:08:42 debug Exp $
  *  
  *  NS16550 serial controller.
  *
@@ -91,6 +91,7 @@ void dev_ns16550_tick(struct cpu *cpu, void *extra)
 	 *  If interrupts are enabled, and interrupts are pending, then
 	 *  cause a CPU interrupt.
  	 */
+
 	if (((d->reg[com_ier] & IER_ETXRDY) && (d->reg[com_iir] & IIR_TXRDY)) ||
 	    ((d->reg[com_ier] & IER_ERXRDY) && (d->reg[com_iir] & IIR_RXRDY))) {
 		d->reg[com_iir] &= ~IIR_NOPEND;
@@ -214,6 +215,8 @@ DEVICE_ACCESS(ns16550)
 			d->fcr = idata;
 		} else {
 			odata = d->reg[com_iir];
+			if (d->reg[com_iir] & IIR_TXRDY)
+				d->reg[com_iir] &= ~IIR_TXRDY;
 			debug("[ ns16550 (%s): read from iir: 0x%02x ]\n",
 			    d->name, (int)odata);
 			dev_ns16550_tick(cpu, d);
@@ -287,6 +290,10 @@ DEVICE_ACCESS(ns16550)
 			d->reg[com_mcr] = idata;
 			debug("[ ns16550 (%s): write to mcr: 0x%02x ]\n",
 			    d->name, (int)idata);
+			if (!(d->reg[com_iir] & IIR_TXRDY)
+			    && (idata & MCR_IENABLE))
+				d->reg[com_iir] |= IIR_TXRDY;
+			dev_ns16550_tick(cpu, d);
 		} else {
 			odata = d->reg[com_mcr];
 			debug("[ ns16550 (%s): read from mcr: 0x%02x ]\n",
@@ -358,7 +365,7 @@ DEVINIT(ns16550)
 	    DEV_NS16550_LENGTH * d->addrmult, dev_ns16550_access, d,
 	    DM_DEFAULT, NULL);
 	machine_add_tickfunction(devinit->machine,
-	    dev_ns16550_tick, d, TICK_SHIFT);
+	    dev_ns16550_tick, d, TICK_SHIFT, 0.0);
 
 	/*
 	 *  NOTE:  Ugly cast into a pointer, because this is a convenient way
