@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_zero.c,v 1.2 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_zero.c,v 1.5 2006/02/09 20:02:59 debug Exp $
  *  
  *  A simple device which returns zero for all reads, and discards all writes.
  */
@@ -40,17 +40,10 @@
 #include "misc.h"
 
 
-/*
- *  dev_zero_access():
- *
- *  Returns 1 if ok, 0 on error.
- */
-int dev_zero_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(zero)
 {
 	if (writeflag == MEM_READ) {
-		int i;
+		unsigned int i;
 		for (i=0; i<len; i++)
 			data[i] = 0;
 	}
@@ -59,10 +52,7 @@ int dev_zero_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_zero():
- */
-int devinit_zero(struct devinit *devinit)
+DEVINIT(zero)
 {
 	memory_device_register(devinit->machine->memory,
 	    devinit->name, devinit->addr, devinit->len,
