@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: diskimage.c,v 1.108 2006/01/11 20:14:41 debug Exp $
+ *  $Id: diskimage.c,v 1.110 2006/06/24 19:52:27 debug Exp $
  *
  *  Disk image support.
  *
@@ -524,7 +524,7 @@ int diskimage_scsicommand(struct cpu *cpu, int id, int type,
 		fatal(" %02x", xferp->cmd[i]);
 	fatal("\n");
 if (xferp->cmd_len > 7 && xferp->cmd[5] == 0x11)
-	single_step = 1;
+	single_step = ENTER_SINGLE_STEPPING;
 #endif
 
 #if 0
@@ -1499,8 +1499,10 @@ int diskimage_add(struct machine *machine, char *fname)
 		d2->next = d;
 	}
 
+	/*  Default to IDE disks...  */
 	d->type = DISKIMAGE_IDE;
 
+	/*  ... but some machines use SCSI by default:  */
 	if (machine->machine_type == MACHINE_PMAX ||
 	    machine->machine_type == MACHINE_ARC)
 		d->type = DISKIMAGE_SCSI;
