@@ -2,7 +2,7 @@
 #define	ARCBIOS_H
 
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: arcbios.h,v 1.10 2005/09/21 19:10:35 debug Exp $
+ *  $Id: arcbios.h,v 1.12 2006/02/02 19:30:32 debug Exp $
  *
  *  Headerfile for src/arcbios.c.
  *
@@ -59,8 +59,8 @@ void arcbios_set_default_exception_handler(struct cpu *cpu);
 
 void arcbios_console_init(struct machine *machine,
 	uint64_t vram, uint64_t ctrlregs);
-void arcbios_init(struct machine *machine, int is64bit,
-	uint64_t sgi_ram_offset);
+void arcbios_init(struct machine *machine, int is64bit, uint64_t sgi_ram_offset,
+	char *primary_ether_string, uint8_t *primary_ether_macaddr);
 
 
 /*  For internal use in arcbios.c:  */
@@ -79,5 +79,7 @@ struct emul_arc_child64 {
 	struct arcbios_component64	component;
 };
 
+#define	ARC_BOOTSTR_BUFLEN		1000
+
 
 #endif	/*  ARCBIOS_H  */
