@@ -2,7 +2,7 @@
 #define	BUS_PCI_H
 
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bus_pci.h,v 1.23 2005/11/23 23:31:37 debug Exp $
+ *  $Id: bus_pci.h,v 1.29 2006/02/18 13:15:21 debug Exp $
  */
 
 #include "misc.h"
@@ -70,8 +70,8 @@ struct pci_data {
 	uint64_t	cur_pci_portbase;
 	uint64_t	cur_pci_membase;
 
-	/*  Current (indirect) addr/data access:  */
-	uint32_t	pci_addr;
+	/*  Current register access:  */
+	int		cur_bus, cur_device, cur_func, cur_reg;
 	int		last_was_write_ffffffff;
 
 	struct pci_device *first_device;
@@ -86,6 +86,7 @@ struct pci_device {
 	int			bus, device, function;
 	unsigned char		cfg_mem[PCI_CFG_MEM_SIZE];
 	unsigned char		cfg_mem_size[PCI_CFG_MEM_SIZE];
+	int			cur_mapreg_offset;
 };
 
 #define	PCIINIT(name)	void pciinit_ ## name(struct machine *machine,	\
@@ -114,15 +115,25 @@ struct pci_device {
 #define	BUS_PCI_DATA	0xcfc
 
 
-/*  bus_pci.c:  */
-int bus_pci_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
-	uint64_t *data, int len, int writeflag, struct pci_data *pci_data);
-void bus_pci_add(struct machine *machine, struct pci_data *pci_data,
-	struct memory *mem, int bus, int device, int function, char *name);
-struct pci_data *bus_pci_init(int irq_nr,
+/*
+ *  bus_pci.c:
+ */
+
+/*  Run-time access:  */
+void bus_pci_decompose_1(uint32_t t, int *bus, int *dev, int *func, int *reg);
+void bus_pci_setaddr(struct cpu *cpu, struct pci_data *pci_data,
+	int bus, int device, int function, int reg);
+void bus_pci_data_access(struct cpu *cpu, struct pci_data *pci_data,
+	uint64_t *data, int len, int writeflag);
+
+/*  Initialization:  */
+struct pci_data *bus_pci_init(struct machine *machine, int irq_nr,
 	uint64_t pci_actual_io_offset, uint64_t pci_actual_mem_offset,
 	uint64_t pci_portbase, uint64_t pci_membase, int pci_irqbase,
 	uint64_t isa_portbase, uint64_t isa_membase, int isa_irqbase);
+void bus_pci_add(struct machine *machine, struct pci_data *pci_data,
+	struct memory *mem, int bus, int device, int function,
+	const char *name);
 
 
 #endif	/*  BUS_PCI_H  */
