@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ia64.h,v 1.10 2006/02/13 04:23:25 debug Exp $
+ *  $Id: cpu_ia64.h,v 1.13 2006/02/24 00:20:42 debug Exp $
  */
 
 #include "misc.h"
@@ -45,7 +45,11 @@ struct cpu_family;
 #define	IA64_ADDR_TO_PAGENR(a)		((a) >> (IA64_IC_ENTRIES_SHIFT \
 					+ IA64_INSTR_ALIGNMENT_SHIFT))
 
+#define	IA64_L2N		17
+#define	IA64_L3N		18
+
 DYNTRANS_MISC_DECLARATIONS(ia64,IA64,uint64_t)
+DYNTRANS_MISC64_DECLARATIONS(ia64,IA64,uint8_t)
 
 #define	IA64_MAX_VPH_TLB_ENTRIES		128
 
@@ -71,6 +75,7 @@ void ia64_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
 void ia64_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void ia64_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
+void ia64_init_64bit_dummy_tables(struct cpu *cpu);
 int ia64_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
 int ia64_userland_memory_rw(struct cpu *cpu, struct memory *mem,
