@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ppc.h,v 1.60 2006/02/09 22:40:27 debug Exp $
+ *  $Id: cpu_ppc.h,v 1.65 2006/06/24 21:47:24 debug Exp $
  */
 
 #include "misc.h"
@@ -84,6 +84,7 @@ struct ppc_cpu_type_def {
 
 #define	PPC_NGPRS		32
 #define	PPC_NFPRS		32
+#define	PPC_NVRS		32
 #define	PPC_N_TGPRS		4
 
 #define	PPC_N_IC_ARGS			3
@@ -95,7 +96,11 @@ struct ppc_cpu_type_def {
 #define	PPC_ADDR_TO_PAGENR(a)		((a) >> (PPC_IC_ENTRIES_SHIFT \
 					+ PPC_INSTR_ALIGNMENT_SHIFT))
 
+#define	PPC_L2N			17
+#define	PPC_L3N			18
+
 DYNTRANS_MISC_DECLARATIONS(ppc,PPC,uint64_t)
+DYNTRANS_MISC64_DECLARATIONS(ppc,PPC,uint8_t)
 
 #define	PPC_MAX_VPH_TLB_ENTRIES		128
 
@@ -117,6 +122,9 @@ struct ppc_cpu {
 	uint64_t	gpr[PPC_NGPRS];	/*  General Purpose Registers  */
 	uint64_t	fpr[PPC_NFPRS];	/*  Floating-Point Registers  */
 
+	uint64_t	vr_hi[PPC_NVRS];/*  128-bit Vector registers  */
+	uint64_t	vr_lo[PPC_NVRS];/*  (Hi and lo 64-bit parts)  */
+
 	uint64_t	msr;		/*  Machine state register  */
 	uint64_t	tgpr[PPC_N_TGPRS];/*Temporary gpr 0..3  */
 
@@ -199,12 +207,13 @@ void ppc_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void ppc32_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void ppc_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 void ppc32_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
+void ppc_init_64bit_dummy_tables(struct cpu *cpu);
 int ppc_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
 int ppc_cpu_family_init(struct cpu_family *);
 
 /*  memory_ppc.c:  */
-int ppc_translate_address(struct cpu *cpu, uint64_t vaddr,
+int ppc_translate_v2p(struct cpu *cpu, uint64_t vaddr,
 	uint64_t *return_addr, int flags);
 
 #endif	/*  CPU_PPC_H  */
