@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_x86.h,v 1.43 2006/02/13 04:23:25 debug Exp $
+ *  $Id: cpu_x86.h,v 1.48 2006/06/24 21:47:24 debug Exp $
  *
  *  x86 (including AMD64) cpu dependent stuff.
  */
@@ -103,7 +103,11 @@ struct x86_model {
 #define	X86_PC_TO_IC_ENTRY(a)		((a) & (X86_IC_ENTRIES_PER_PAGE-1))
 #define	X86_ADDR_TO_PAGENR(a)		((a) >> X86_IC_ENTRIES_SHIFT)
 
+#define	X86_L2N			17
+#define	X86_L3N			18
+
 DYNTRANS_MISC_DECLARATIONS(x86,X86,uint64_t)
+DYNTRANS_MISC64_DECLARATIONS(x86,X86,uint8_t)
 
 #define	X86_MAX_VPH_TLB_ENTRIES		128
 
@@ -282,9 +286,11 @@ struct x86_cpu {
 #define	DESCR_TYPE_DATA		2
 
 
+#define	LONG_MODE		(cpu->cd.x86.efer & X86_EFER_LME)
 #define	PROTECTED_MODE		(cpu->cd.x86.cr[0] & X86_CR0_PE)
 #define	REAL_MODE		(!PROTECTED_MODE)
 
+
 /*  cpu_x86.c:  */
 void reload_segment_descriptor(struct cpu *cpu, int segnr, int selector,
 	uint64_t *curpcp);
@@ -299,11 +305,12 @@ void x86_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void x8632_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void x86_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
 void x8632_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
+void x86_init_64bit_dummy_tables(struct cpu *cpu);
 int x86_cpu_family_init(struct cpu_family *);
 
 
 /*  memory_x86.c:  */
-int x86_translate_address(struct cpu *cpu, uint64_t vaddr,
+int x86_translate_v2p(struct cpu *cpu, uint64_t vaddr,
 	uint64_t *return_addr, int flags);
 
 #endif	/*  CPU_X86_H  */
