@@ -2,7 +2,7 @@
 #define	DEBUGGER_H
 
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,12 +28,13 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: debugger.h,v 1.2 2006/02/04 11:10:59 debug Exp $
+ *  $Id: debugger.h,v 1.5 2006/06/24 19:52:28 debug Exp $
  *
- *  See src/debugger.c.
+ *  See src/debugger/debugger.c.
  */
 
 struct emul;
+struct machine;
 
 /*  debugger.c:  */
 void debugger_activate(int x);
@@ -42,5 +43,9 @@ void debugger(void);
 void debugger_reset(void);
 void debugger_init(struct emul **emuls, int n_emuls);
 
+/*  single_step values:  */
+#define	NOT_SINGLE_STEPPING		0
+#define	ENTER_SINGLE_STEPPING		1
+#define	SINGLE_STEPPING			2
 
 #endif	/*  DEBUGGER_H  */
