@@ -2,7 +2,7 @@
 #define	DEVICES_H
 
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: devices.h,v 1.207 2006/02/18 17:55:25 debug Exp $
+ *  $Id: devices.h,v 1.212 2006/06/16 18:31:26 debug Exp $
  *
  *  Memory mapped devices.
  *
@@ -148,10 +148,6 @@ void dev_bt459_init(struct machine *machine, struct memory *mem,
 	int color_fb_flag, int irq_nr, int type);
 
 /*  dev_cons.c:  */
-#define	DEV_CONS_ADDRESS		0x0000000010000000
-#define	DEV_CONS_LENGTH			0x0000000000000020
-#define	    DEV_CONS_PUTGETCHAR		    0x0000
-#define	    DEV_CONS_HALT		    0x0010
 struct cons_data {
 	int	console_handle;
 	int	irq_nr;
@@ -201,19 +197,11 @@ void dev_deccca_init(struct memory *mem, uint64_t baseaddr);
 int dev_decxmi_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 void dev_decxmi_init(struct memory *mem, uint64_t baseaddr);
 
-/*  dev_disk.c:  */
-#define	DEV_DISK_ADDRESS		0x13000000
-
 /*  dev_eagle.c:  */
 struct pci_data *dev_eagle_init(struct machine *machine, struct memory *mem,
 	int irqbase, int pciirq);
 
-/*  dev_ether.c:  */
-#define	DEV_ETHER_ADDRESS		0x14000000
-#define	DEV_ETHER_LENGTH		0x8000
-
 /*  dev_fb.c:  */
-#define	DEV_FB_ADDRESS		0x12000000	/*  Default for testmips  */
 #define	DEV_FB_LENGTH		0x3c0000	/*  3c0000 to not colide with */
 						/*  turbochannel rom,         */
 						/*  otherwise size = 4MB      */
@@ -235,7 +223,7 @@ struct vfb_data {
 	int		ysize;
 	int		bit_depth;
 	int		color32k;	/*  hack for 16-bit HPCmips  */
-	int		psp_15bit;	/*  plastation portable hack  */
+	int		psp_15bit;	/*  playstation portable hack  */
 
 	unsigned char	color_plane_mask;
 
@@ -382,7 +370,7 @@ void dev_vdac_init(struct memory *mem, uint64_t baseaddr, unsigned char *rgb_pal
 /*  dev_kn02.c:  */
 struct kn02_csr {
 	uint8_t		csr[sizeof(uint32_t)];
-	uint8_t		filler[4096 - sizeof(uint32_t)];  /*  for bintrans mapping  */
+	uint8_t		filler[4096 - sizeof(uint32_t)];  /*  for dyntrans mapping  */
 };
 int dev_kn02_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, unsigned char *data, size_t len, int writeflag, void *);
 struct kn02_csr *dev_kn02_init(struct cpu *cpu, struct memory *mem,
