@@ -2,7 +2,7 @@
 #define	MEMORY_H
 
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory.h,v 1.47 2006/01/01 13:17:18 debug Exp $
+ *  $Id: memory.h,v 1.50 2006/06/24 21:47:24 debug Exp $
  *
  *  Memory controller related functions.
  */
@@ -45,9 +45,6 @@
 struct cpu;
 struct translation_page_entry;
 
-/*  For bintrans:  */
-#define	MAX_QUICK_JUMPS			8
-
 struct memory {
 	uint64_t	physical_max;
 	void		*pagetable;
@@ -73,35 +70,6 @@ struct memory {
 	uint64_t	dev_dyntrans_write_high[MAX_DEVICES];
 
 	int		dev_dyntrans_alignment;
-
-
-	/*
-	 *  NOTE/TODO: This bintrans was for MIPS only. Ugly. :-/
-	 */
-
-	/*
-	 *  translation_code_chunk_space is a large chunk of (linear) memory
-	 *  where translated code chunks and translation_entrys are stored.
-	 *  When this is filled, translation is restart from scratch (by
-	 *  resetting translation_code_chunk_space_head to 0, and removing all
-	 *  translation entries).
-	 *
-	 *  (Using a static memory region like this is somewhat inspired by
-	 *  the QEMU web pages,
-	 *  http://fabrice.bellard.free.fr/qemu/qemu-tech.html#SEC13)
-	 */
-
-	unsigned char	*translation_code_chunk_space;
-	size_t		translation_code_chunk_space_head;
-
-	int		bintrans_32bit_only;
-
-	struct translation_page_entry **translation_page_entry_array;
-
-	unsigned char	*quick_jump_host_address[MAX_QUICK_JUMPS];
-	size_t		quick_jump_page_offset[MAX_QUICK_JUMPS];
-	int		n_quick_jumps;
-	int		quick_jumps_index;
 };
 
 #define	BITS_PER_PAGETABLE	20
@@ -127,22 +95,6 @@ char *memory_conv_to_string(struct cpu *cpu, struct memory *mem,
 unsigned char *memory_paddr_to_hostaddr(struct memory *mem,
 	uint64_t paddr, int writeflag);
 
-/*  memory_fast_v2h.c:  */
-unsigned char *fast_vaddr_to_hostaddr(struct cpu *cpu, uint64_t vaddr,
-	int writeflag);
-
-/*  MIPS stuff:  */
-int translate_address_mmu3k(struct cpu *cpu, uint64_t vaddr,
-	uint64_t *return_addr, int flags);
-int translate_address_mmu8k(struct cpu *cpu, uint64_t vaddr,
-	uint64_t *return_addr, int flags);
-int translate_address_mmu10k(struct cpu *cpu, uint64_t vaddr,
-	uint64_t *return_addr, int flags);
-int translate_address_mmu4100(struct cpu *cpu, uint64_t vaddr,
-	uint64_t *return_addr, int flags);
-int translate_address_generic(struct cpu *cpu, uint64_t vaddr,
-	uint64_t *return_addr, int flags);
-
 
 /*  Writeflag:  */
 #define	MEM_READ			0
@@ -190,4 +142,7 @@ void memory_device_register(struct memory *mem, const char *,
 	void *extra, int flags, unsigned char *dyntrans_data);
 void memory_device_remove(struct memory *mem, int i);
 
+uint64_t memory_checksum(struct memory *mem);
+
+
 #endif	/*  MEMORY_H  */
