@@ -2,7 +2,7 @@
 #define	MISC_H
 
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: misc.h,v 1.236 2005/12/03 04:14:16 debug Exp $
+ *  $Id: misc.h,v 1.241 2006/06/16 18:31:26 debug Exp $
  *
  *  Misc. definitions for gxemul.
  */
@@ -45,14 +45,28 @@
 
 #include "../../config.h"
 
-/*  
- *  ENABLE_INSTRUCTION_DELAYS should be defined on the cc commandline using
- *  -D if you want it. (This is done by ./configure --delays)
- *
- *  ALWAYS_SIGNEXTEND_32 is enabled by ./configure --always32
+
+#ifdef NO_C99_PRINTF_DEFINES
+/*
+ *  This is a SUPER-UGLY HACK which happens to work on some machines.
+ *  The correct solution is to upgrade your compiler to C99.
  */
-#define USE_TINY_CACHE
-/*  #define HALT_IF_PC_ZERO  */
+#ifdef NO_C99_64BIT_LONGLONG
+#define	PRIi16		"i"
+#define	PRIi32		"i"
+#define	PRIi64		"lli"
+#define	PRIx16		"x"
+#define	PRIx32		"x"
+#define	PRIx64		"llx"
+#else
+#define	PRIi16		"i"
+#define	PRIi32		"i"
+#define	PRIi64		"li"
+#define	PRIx16		"x"
+#define	PRIx32		"x"
+#define	PRIx64		"lx"
+#endif
+#endif
 
 
 #ifdef NO_MAP_ANON
@@ -110,9 +124,6 @@ struct memory;
 /*  Debug stuff:  */
 #define	DEBUG_BUFSIZE		1024
 #define	DEBUG_INDENTATION	4
-#ifndef	DEFAULT_BINTRANS_SIZE_IN_MB
-#define	DEFAULT_BINTRANS_SIZE_IN_MB	16
-#endif
 
 
 /*  dec_prom.c:  */
