@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: interrupts.c,v 1.6 2006/02/18 13:42:39 debug Exp $
+ *  $Id: interrupts.c,v 1.9 2006/05/17 20:27:31 debug Exp $
  *
  *  Machine-dependent interrupt glue.
  */
@@ -337,7 +337,7 @@ void vr41xx_interrupt(struct machine *m, struct cpu *cpu, int irq_nr, int assrt)
 void ps2_interrupt(struct machine *m, struct cpu *cpu, int irq_nr, int assrt)
 {
 	irq_nr -= 8;
-	debug("ps2_interrupt(): irq_nr=0x%x assrt=%i\n", irq_nr, assrt);
+	/*  debug("ps2_interrupt(): irq_nr=0x%x assrt=%i\n", irq_nr, assrt);  */
 
 	if (irq_nr >= 32) {
 		int msk = 0;
@@ -935,7 +935,7 @@ void gc_interrupt(struct machine *m, struct cpu *cpu, int irq_nr, int assrt)
 			m->md_int.gc_data->status_hi &= ~mask;
 	}
 
-#if 1
+#if 0
 	printf("status = %08x %08x  enable = %08x %08x\n",
 	    m->md_int.gc_data->status_hi, m->md_int.gc_data->status_lo,
 	    m->md_int.gc_data->enable_hi, m->md_int.gc_data->enable_lo);
@@ -964,7 +964,7 @@ void i80321_interrupt(struct machine *m, struct cpu *cpu, int irq_nr, int assrt)
 			cpu->cd.arm.i80321_isrc &= ~mask;
 	}
 
-	/*  debug("isrc = %08x  inten = %08x\n",
+	/*  fatal("isrc = %08x  inten = %08x\n",
 	    cpu->cd.arm.i80321_isrc, cpu->cd.arm.i80321_inten);  */
 
 	if (cpu->cd.arm.i80321_isrc & cpu->cd.arm.i80321_inten)
