@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: main.c,v 1.256 2006/02/04 11:10:58 debug Exp $
+ *  $Id: main.c,v 1.275 2006/06/24 19:52:28 debug Exp $
  */
 
 #include <stdio.h>
@@ -37,26 +37,28 @@
 
 #include "console.h"
 #include "cpu.h"
+#include "debugger.h"
 #include "device.h"
 #include "diskimage.h"
 #include "emul.h"
 #include "machine.h"
 #include "misc.h"
+#include "settings.h"
 
 
 extern volatile int single_step;
 extern int force_debugger_at_exit;
-extern int show_opcode_statistics;
 
 extern int optind;
 extern char *optarg;
 
+struct settings *global_settings;
+
 int extra_argc;
 char **extra_argv;
 char *progname;
 
 int fully_deterministic = 0;
-int dyntrans_backend_enable = 1;
 
 
 /*****************************************************************************
@@ -202,8 +204,10 @@ static void usage(int longusage)
 	printf("\nusage: %s [machine, other, and general options] [file "
 	    "[...]]\n", progname);
 	printf("   or  %s [general options] @configfile\n", progname);
+#ifdef UNSTABLE_DEVEL
 	printf("   or  %s [userland, other, and general options] file "
 	    "[args ...]\n", progname);
+#endif
 
 	if (!longusage) {
 		printf("\nRun  %s -h  for help on command line options.\n",
@@ -220,11 +224,6 @@ static void usage(int longusage)
 	printf("\nOther options:\n");
 	printf("  -A        disable alignment checks in some cases (for higher"
 	    " speed)\n");
-#if defined(BINTRANS) || defined(DYNTRANS_BACKEND)
-	printf("  -B        disable native translation backends. (translation"
-	    " is turned on\n            by default, if it is supposed for "
-	    "the particular host)\n");
-#endif
 	printf("  -C x      try to emulate a specific CPU. (Use -H to get a "
 	    "list of types.)\n");
 	printf("  -d fname  add fname as a disk image. You can add \"xxx:\""
@@ -243,27 +242,33 @@ static void usage(int longusage)
 	printf("                s      SCSI\n");
 	printf("                t      tape\n");
 	printf("                0-7    force a specific ID\n");
+	printf("  -G port   listen to gdb remote connections on this port\n");
 	printf("  -I x      emulate clock interrupts at x Hz (affects"
 	    " rtc devices only, not\n");
 	printf("            actual runtime speed) (this disables automatic"
 	    " clock adjustments)\n");
 	printf("  -i        display each instruction as it is executed\n");
 	printf("  -J        disable some speed tricks\n");
-	printf("  -j name   set the name of the kernel, for example:\n");
-	printf("                -j netbsd          for NetBSD/pmax\n");
-	printf("                -j bsd             for OpenBSD/pmax\n");
-	printf("                -j vmunix          for Ultrix/RISC\n");
+	printf("  -j name   set the name of the kernel; for DECstation "
+	    "emulation, this passes\n            the name to the bootloader,"
+	    " for example:\n");
+	printf("                -j netbsd     (NetBSD/pmax)      "
+	    "-j bsd      (OpenBSD/pmax)\n");
+	printf("                -j vmsprite   (Sprite/pmax)      "
+	    "-j vmunix   (Ultrix/RISC)\n");
+	printf("            For other emulation modes, if the boot disk is an"
+	    " ISO9660\n            filesystem, -j sets the name of the"
+	    " kernel to load.\n");
 	printf("  -M m      emulate m MBs of physical RAM\n");
-	printf("  -m nr     run at most nr instructions (on any cpu)\n");
 	printf("  -N        display nr of instructions/second average, at"
 	    " regular intervals\n");
 	printf("  -n nr     set nr of CPUs (for SMP experiments)\n");
 	printf("  -O        force netboot (tftp instead of disk), even when"
 	    " a disk image is\n"
 	    "            present (for DECstation, SGI, and ARC emulation)\n");
-	printf("  -o arg    set the boot argument (for DEC, ARC, or SGI"
-	    " emulation).\n");
-	printf("            Default arg for DEC is '-a', for ARC '-aN'.\n");
+	printf("  -o arg    set the boot argument, for DEC, ARC, or SGI"
+	    " emulation\n");
+	printf("            (default arg for DEC is -a, for ARC/SGI -aN)\n");
 	printf("  -p pc     add a breakpoint (remember to use the '0x' "
 	    "prefix for hex!)\n");
 	printf("  -Q        no built-in PROM emulation  (use this for "
@@ -272,42 +277,38 @@ static void usage(int longusage)
 	printf("  -r        register dumps before every instruction\n");
 	printf("  -S        initialize emulated RAM to random bytes, "
 	    "instead of zeroes\n");
-	printf("  -T        enter the single-step debugger on "
-	    "unimplemented memory accesses\n");
 	printf("  -t        show function trace tree\n");
 	printf("  -U        enable slow_serial_interrupts_hack_for_linux\n");
 #ifdef WITH_X11
 	printf("  -X        use X11\n");
-#endif /*  WITH_X11  */
 	printf("  -x        open up new xterms for emulated serial ports "
 	    "(default is on when\n            using configuration files or"
 	    " when X11 is used, off otherwise)\n");
-#ifdef WITH_X11
 	printf("  -Y n      scale down framebuffer windows by n x n times\n");
 #endif /*  WITH_X11  */
-	printf("  -y x      set max_random_cycles_per_chunk to x"
-	    " (experimental)\n");
 	printf("  -Z n      set nr of graphics cards, for emulating a "
 	    "dual-head or tripple-head\n"
 	    "            environment (only for DECstation emulation)\n");
 	printf("  -z disp   add disp as an X11 display to use for "
 	    "framebuffers\n");
 
+#ifdef UNSTABLE_DEVEL
 	printf("\nUserland options:\n");
 	printf("  -u emul   userland-only (syscall) emulation (use -H to"
 	    " get a list of\n            available emulation modes)\n");
+#endif
 
 	printf("\nGeneral options:\n");
 	printf("  -c cmd    add cmd as a command to run before starting "
 	    "the simulation\n");
-	printf("  -D        guarantee fully deterministic behaviour\n");
+	printf("  -D        guarantee (almost) fully deterministic "
+	    "behaviour\n");
 	printf("  -H        display a list of possible CPU and "
 	    "machine types\n");
 	printf("  -h        display this help message\n");
 	printf("  -K        force the debugger to be entered at the end "
 	    "of a simulation\n");
 	printf("  -q        quiet mode (don't print startup messages)\n");
-	printf("  -s        show opcode usage statistics after simulation\n");
 	printf("  -V        start up in the single-step debugger, paused\n");
 	printf("  -v        verbose debug messages\n");
 	printf("\n");
@@ -338,24 +339,32 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 	char ***diskimagesp, int *n_diskimagesp)
 {
 	int ch, res, using_switch_d = 0, using_switch_Z = 0;
+	int using_switch_e = 0, using_switch_E = 0;
 	char *type = NULL, *subtype = NULL;
 	int n_cpus_set = 0;
 	int msopts = 0;		/*  Machine-specific options used  */
 	struct machine *m = emul_add_machine(emul, "default");
 
-	while ((ch = getopt(argc, argv, "ABC:c:Dd:E:e:HhI:iJj:KM:m:"
-	    "Nn:Oo:p:QqRrSsTtUu:VvW:XxY:y:Z:z:")) != -1) {
+	char *opts =
+	    "A"
+	    "C:c:Dd:E:e:G:HhI:iJj:KM:Nn:Oo:p:QqRrStU"
+#ifdef UNSTABLE_DEVEL
+	    "u:"
+#endif
+	    "VvW:"
+#ifdef WITH_X11
+	    "XxY:"
+#endif
+	    "Z:z:";
+
+	while ((ch = getopt(argc, argv, opts)) != -1) {
 		switch (ch) {
 		case 'A':
+			fprintf(stderr, "NOTE: The -A command line option"
+			    " is DEPRECATED and will be removed soon.\n");
 			m->dyntrans_alignment_check = 0;
 			msopts = 1;
 			break;
-		case 'B':
-			/*  Turns off both bintrans and dyntrans.  */
-			m->bintrans_enable = 0;
-			dyntrans_backend_enable = 0;
-			msopts = 1;
-			break;
 		case 'C':
 			m->cpu_name = strdup(optarg);
 			msopts = 1;
@@ -388,13 +397,32 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			msopts = 1;
 			break;
 		case 'E':
+			if (using_switch_E ++ > 0) {
+				fprintf(stderr, "-E already used.\n");
+				exit(1);
+			}
 			type = optarg;
 			msopts = 1;
 			break;
 		case 'e':
+			if (using_switch_e ++ > 0) {
+				fprintf(stderr, "-e already used.\n");
+				exit(1);
+			}
 			subtype = optarg;
 			msopts = 1;
 			break;
+		case 'G':
+			m->gdb.port = atoi(optarg);
+			if (m->gdb.port < 1 || m->gdb.port > 65535) {
+				fprintf(stderr, "Invalid debugger port %i.\n",
+				    m->gdb.port);
+				exit(1);
+			}
+			/*  Note: implicit -V  */
+			single_step = ENTER_SINGLE_STEPPING;
+			msopts = 1;
+			break;
 		case 'H':
 			machine_list_available_types_and_cpus();
 			exit(1);
@@ -429,10 +457,6 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			m->physical_ram_in_mb = atoi(optarg);
 			msopts = 1;
 			break;
-		case 'm':
-			m->max_instructions = atoi(optarg);
-			msopts = 1;
-			break;
 		case 'N':
 			m->show_nr_of_instructions = 1;
 			msopts = 1;
@@ -487,13 +511,6 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			m->random_mem_contents = 1;
 			msopts = 1;
 			break;
-		case 's':
-			show_opcode_statistics = 1;
-			break;
-		case 'T':
-			m->single_step_on_bad_addr = 1;
-			msopts = 1;
-			break;
 		case 't':
 			m->show_trace_tree = 1;
 			msopts = 1;
@@ -512,7 +529,7 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			msopts = 1;
 			break;
 		case 'V':
-			single_step = 1;
+			single_step = ENTER_SINGLE_STEPPING;
 			break;
 		case 'v':
 			verbose ++;
@@ -539,10 +556,6 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 			}
 			msopts = 1;
 			break;
-		case 'y':
-			m->max_random_cycles_per_chunk = atoi(optarg);
-			msopts = 1;
-			break;
 		case 'Z':
 			m->n_gfx_cards = atoi(optarg);
 			using_switch_Z = 1;
@@ -614,13 +627,6 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 		quiet_mode = 0;
 	}
 
-	if ((m->instruction_trace || m->register_dump || m->show_trace_tree)
-	    && m->bintrans_enable) {
-		if (m->arch == ARCH_MIPS)
-			fprintf(stderr, "Implicitly turning off bintrans.\n");
-		m->bintrans_enable = 0;
-	}
-
 
 	/*
 	 *  Usually, an executable filename must be supplied.
@@ -667,41 +673,9 @@ int get_cmd_args(int argc, char *argv[], struct emul *emul,
 		exit(1);
 	}
 
-#ifndef EXPERIMENTAL_NEWMIPS
-	if (m->bintrans_enable && m->arch == ARCH_MIPS) {
-		m->speed_tricks = 0;
-		/*  TODO: Print a warning about this?  */
-	}
-#endif
-
-	if (m->n_breakpoints > 0 &&
-	    m->bintrans_enable && m->arch == ARCH_MIPS) {
-		fprintf(stderr, "Breakpoints and MIPS binary translation "
-		    "don't work too well together right now.\n");
-		exit(1);
-	}
-
-#ifndef BINTRANS
-	if (m->bintrans_enable) {
-		fprintf(stderr, "WARNING: %s was compiled without "
-		    "bintrans support. Ignoring -b.\n", progname);
-		m->bintrans_enable = 0;
-	}
-#endif
-
-#ifndef WITH_X11
-	if (m->use_x11) {
-		fprintf(stderr, "WARNING: %s was compiled without "
-		    "X11 support. Ignoring -X.\n", progname);
-		m->use_x11 = 0;
-	}
-#endif
-
 	if (!using_switch_Z && !m->use_x11)
 		m->n_gfx_cards = 0;
 
-	m->bintrans_enabled_from_start = m->bintrans_enable;
-
 	return 0;
 }
 
@@ -726,6 +700,23 @@ int main(int argc, char *argv[])
 
 	progname = argv[0];
 
+	/*  Create the settings object, and add global settings to it:  */
+	global_settings = settings_new();
+
+	settings_add(global_settings, "single_step", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO, (void *)&single_step);
+	settings_add(global_settings, "force_debugger_at_exit", 1,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
+	    (void *)&force_debugger_at_exit);
+	settings_add(global_settings, "fully_deterministic", 0,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO,
+	    (void *)&fully_deterministic);
+	settings_add(global_settings, "verbose", 1,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO, (void *)&verbose);
+	settings_add(global_settings, "quiet_mode", 1,
+	    SETTINGS_TYPE_INT, SETTINGS_FORMAT_YESNO, (void *)&quiet_mode);
+
+	/*  Initialize all emulator subsystems:  */
 	console_init();
 	cpu_init();
 	device_init();
@@ -832,6 +823,8 @@ int main(int argc, char *argv[])
 	/*  Run all emulations:  */
 	emul_run(emuls, n_emuls);
 
+	settings_destroy(global_settings);
+
 	return 0;
 }
 
