@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: net.c,v 1.84 2005/11/25 03:38:00 debug Exp $
+ *  $Id: net.c,v 1.89 2006/02/09 22:40:26 debug Exp $
  *
  *  Emulated (ethernet / internet) network support.
  *
@@ -157,20 +157,13 @@ void net_generate_unique_mac(struct machine *machine, unsigned char *macbuf)
 	x = machine->serial_nr;
 	y = machine->nr_of_nics;
 
-	/*
-	 *  TODO: What is a good starting value? Something like this?
-	 *
-	 *  +-----------+-------------------+-------------+-------------+
-	 *  |  16 bits  |  16 bits machine  |  12 bits    |  4 bits of  |
-	 *  |  fixed    |  serial nr        |  NIC nr(*)  |  zeroes     |
-	 *  +-----------+-------------------+-------------+-------------+
-	 */
 	macbuf[0] = 0x10;
 	macbuf[1] = 0x20;
 	macbuf[2] = 0x30;
 	macbuf[3] = 0;
 	macbuf[4] = 0;
-	macbuf[5] = machine->serial_nr << 4;
+	/*  NOTE/TODO: This only allows 8 nics per machine!  */
+	macbuf[5] = (machine->serial_nr << 4) + (machine->nr_of_nics << 1);
 
 	if (macbuf[0] & 1 || macbuf[5] & 1) {
 		fatal("Internal error in net_generate_unique_mac().\n");
@@ -1961,7 +1954,7 @@ void send_udp(struct in_addr *addrp, int portnr, unsigned char *packet,
 	si.sin_port = htons(portnr);
 
 	if (sendto(s, packet, len, 0, (struct sockaddr *)&si,
-	    sizeof(si)) != len) {
+	    sizeof(si)) != (ssize_t)len) {
 		perror("send_udp(): sendto");
 	}
 
@@ -2258,7 +2251,7 @@ static void net_gateway_init(struct net *net)
  */
 void net_dumpinfo(struct net *net)
 {
-	int iadd = 4;
+	int iadd = DEBUG_INDENTATION;
 	struct remote_net *rnp;
 
 	debug("net: simulating ");
@@ -2426,7 +2419,7 @@ struct net *net_init(struct emul *emul, int init_flags,
 
 	net_dumpinfo(net);
 
-	/*  This is neccessary when using the real network:  */
+	/*  This is necessary when using the real network:  */
 	signal(SIGPIPE, SIG_IGN);
 
 	return net;
